// nazwa pakietu
package pl.edu.pw.ii.scproj.samples;

// import niezbdnych pakietw
import javacard.framework.*;
import javacard.security.*;
import javacardx.crypto.*;

// klasa LoyaltyExample realizujca funkcjonalno programu lojalnociowego
public class LoyaltyExample extends Applet
{
    // stae wykorzystywane przy analizie komend APDU
    
    // klasa komend APDU apletu lojalnociowego
    private final static byte CLA_LOYALTY     = (byte) 0x90;
    // pobranie informacji o iloci punktw
    private final static byte CMD_GET_POINTS  = (byte) 0x60;
    // dodanie punktw
    private final static byte CMD_ADD_POINTS  = (byte) 0x62;
    // odjcie punktw
    private final static byte CMD_SUB_POINTS  = (byte) 0x64;
    // weryfikacja kodu PIN
    private final static byte CMD_VERIFY_PIN  = (byte) 0x66;
    // weryfikacja kodu PIN
    private final static byte CMD_ASK_RANDOM  = (byte) 0x68;
    // uwierzytelnienie karty 
    private final static byte CMD_INT_AUTH    = (byte) 0x6A;
    // uwierzytelnienie terminala
    private final static byte CMD_EXT_AUTH    = (byte) 0x6C;

    // kod odpowiedzi po niepoprawnym wprowadzeniu kodu PIN
    private final static short SW_WRONG_PIN = (short) 0x63C0;

    // maksymalna warto iloci punktw 
    private static final short maxPoints = 15000;

    // aktualne iloci punktw u partnerw
    private short [] points;

    // iloci punktw u partnerw zgromadzone od pocztku programu
    private short allPoints;

    // kod PIN
    private OwnerPIN ownerPIN;

    // klucz (obustronne uwierzytelnienie)
    private Key desKey;

    // szyfrowanie algorytmem DES
    private Cipher cipherDES;

    // generator liczb losowych
    private RandomData random;

    // flaga okrelajca prawa dostpu 
    private boolean externalAuth;

    // flaga okrelajca czy wygenerowano liczb losow
    private boolean randomGen;

    // bufor zawierajcy ostatnio wygenerowan liczb losow
    private byte[] lastRandom;
   
    // stworzenie instancji apletu na karcie
    private LoyaltyExample(byte[] buffer, short offset, byte length)
    {
        short paramOffset = offset;

	desKey = null; 
	cipherDES = null;

        if(length > 9) 
	{
            // parametry instalacyjne przekazane zgodnie z OpenPlatform 2.0.1

	    // przesunicie do danych okrelajcych prawa apletu
            paramOffset += (short)(1 + buffer[offset]);
	    // przesunicie do parametrw aplikacji
            paramOffset += (short)(1 + buffer[paramOffset]);

	    // oczekiwany jest PIN o dugoci 4 bajtw oraz klucz DES o dugoci 8 bajtw
            if(buffer[paramOffset] != 12)
                // operacja koczy si z bdem
                ISOException.throwIt(
	        (short)(ISO7816.SW_WRONG_LENGTH+offset+length-paramOffset));
	    
            // dane aplikacji (kod PIN, klucz)
            paramOffset++;
        } 
	else 
	{
            // parametry instalacyjne przekazane zgodnie z OpenPlatform 2.0.
	    // oczekiwany jest PIN o dugoci 4 bajtw oraz klucz DES o dugoci 8 bajtw
	    if(length != 12)
		ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + length));
        } 
	
	// stworzenie instancji klasy OwnerPIN: kod PIN o dugoci 4, ilo prb wprowadzenia - 3
	ownerPIN = new OwnerPIN ((byte)3, (byte)4);

	// ustalenie nowego kodu PIN (na podstawie danych podanych przy tworzeniu instancji)
	ownerPIN.update(buffer, paramOffset,(byte)4);

	// inicjalizacja klucza DES
	paramOffset+=4;
	desKey = KeyBuilder.buildKey(KeyBuilder.TYPE_DES, 
				     KeyBuilder.LENGTH_DES, false);
	((DESKey)desKey).setKey(buffer, paramOffset);

	// pocztkowa ilo punktw
        allPoints = 0;
	points = new short[2];
	points[0]=points[1]=0;

	// bufor na liczb losow
	lastRandom = new byte[8];

	// generator liczb losowych
	random = RandomData.getInstance(RandomData.ALG_SECURE_RANDOM);
	
	// instancja klasy szyfrujcej algorytmem DES
	cipherDES = Cipher.getInstance(Cipher.ALG_DES_ECB_NOPAD, false);

	// ustalenie wartoci pocztkowej flag
	resetSecurity();
 
	// rejestracja instancji apletu
	register(buffer, (short)(offset+1) ,(byte)(buffer[offset]));
    }

    // stworzenie instancji apletu
    public static void install(byte[] bArray, short bOffset, byte bLength) 
	throws ISOException
    {
	// wywoanie konstruktora
        new LoyaltyExample (bArray, bOffset, (byte)bLength);
    }

    // wywoywane po wybraniu apletu
    public boolean select()
    {
	// ustalenie wartoci pocztkowej flag
	resetSecurity();
        return true;
    }

    // wywoywane po zakoczeniu korzystania z apletu
    public void deselect()
    {
	// ustalenie wartoci pocztkowej flag
	resetSecurity();
    }

    private void resetSecurity()
    {
	// zerowanie statusu kodu PIN
	ownerPIN.reset();
	// nie byo prawidowego uwierzytelnienia terminala
	externalAuth=false;
	// nie wygenerowano liczby losowej
	randomGen=false;
    }

    // przetwarzanie komend APDU 
    public void process(APDU apdu) throws ISOException
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
	// ilo danych w buforze
        byte dataLength = apduBuffer[ISO7816.OFFSET_LC];

	// przechwycienie komendy SELECT FILE
	if (selectingApplet()) 
            ISOException.throwIt(ISO7816.SW_NO_ERROR);

	// nie obsugujemy kanaw logicznych 
	apduBuffer[ISO7816.OFFSET_CLA] =
           (byte)(apduBuffer[ISO7816.OFFSET_CLA] & (byte)0xFC);
	
        // sprawdzenie poprawnoci klasy instrukcji
        if (apduBuffer[ISO7816.OFFSET_CLA] != CLA_LOYALTY)
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);

	// w zalenoci od przesanej instrukcji podejmowane s rne operacje
	switch(apdu.getBuffer()[ISO7816.OFFSET_INS])
	{
	    // pobranie iloci gotwki w portfelu
	    case CMD_GET_POINTS:
		getPoints(apdu);
		break;

	    // doadowanie
	    case CMD_ADD_POINTS:
		creditLoyalty(apdu);
		break;

	    // transakcja
	    case CMD_SUB_POINTS:
		debitLoyalty(apdu);
		break;

	    // weryfikacja kodu PIN
	    case CMD_VERIFY_PIN:
		verifyPIN(apdu);
		break;

	    // generacja liczby pseudolosowej
	    case CMD_ASK_RANDOM:
		askRandom(apdu);
		break;
	    
	    // uwierzytelnienie karty
	    case CMD_INT_AUTH:
		internalAuth(apdu);
		break;

	    // uwierzytelnienie terminala
	    case CMD_EXT_AUTH:
		externalAuth(apdu);
		break;

	    // kada inna instrukcja jest niepoprawna
	    default:
		// zgoszenie wyjtku
		ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
		break;
	}
    }

    // zwraca aktualn ilo punktw w programie P1 (warto 1 lub 2)
    // lub sum zebranych punktw P1 = 3
    private void getPoints(APDU apdu)
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
            
        // wpisanie aktualnej iloci punktw 
	if (apduBuffer[ISO7816.OFFSET_P1] == 1 ||
	    apduBuffer[ISO7816.OFFSET_P1] == 2)
	{
	    // partner 1 lub 2
	    apduBuffer[5] = (byte)(points[apduBuffer[ISO7816.OFFSET_P1]-1] >> 8);
	    apduBuffer[6] = (byte)points[apduBuffer[ISO7816.OFFSET_P1]-1];
	}
	else
	if (apduBuffer[ISO7816.OFFSET_P1] == 3)
	{
	    // suma punktw
	    apduBuffer[5] = (byte)(allPoints >> 8);
	    apduBuffer[6] = (byte)allPoints;
	}
	else
	    // bdny parametr
	    ISOException.throwIt(ISO7816.SW_INCORRECT_P1P2);

	// przejcie w tryb odpowiedzi
        apdu.setOutgoing(); 
	// dugo odpowiedzi
        apdu.setOutgoingLength((short)2); 

        // przesanie odpowiedzi APDU (2 bajty od bajtu 5 z bufora)
        apdu.sendBytes((short)5, (short)2);
    }

    // dodanie punktw dla partnera P1 (warto 1 lub 2)
    private void creditLoyalty(APDU apdu) throws ISOException
    {
	// terminal nie by uwierzytelniony
        if (!externalAuth)
            ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);

        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
	
	// sprawdzenie poprawnoci numeru partnera
	if (apduBuffer[ISO7816.OFFSET_P1] != 1 &&
	    apduBuffer[ISO7816.OFFSET_P1] != 2)
	    ISOException.throwIt(ISO7816.SW_INCORRECT_P1P2);

        // sprawdzenie czy dostarczono 2 bajty danych (ilo punktw)
        if( apduBuffer[4] != 2 || apdu.setIncomingAndReceive() != 2 )
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

        // odczytanie wartoci doadowania (bajty 5 i 6 w przesaniej komendzie APDU)
        short newPoints = (short)(((apduBuffer[5] << 8 ) & (short)0xFF00) +
	                  ((apduBuffer[6]) & (short)0x00FF));

        // spawdzenie poprawnoci wartoci doadowania
        if ((newPoints>(short)(maxPoints-points[apduBuffer[ISO7816.OFFSET_P1]-1])) 
		|| (newPoints <= (short)0))
	    ISOException.throwIt(ISO7816.SW_DATA_INVALID);
        else
	{
	    // doadowanie punktw
            points[apduBuffer[ISO7816.OFFSET_P1]-1] += newPoints;
	    allPoints += newPoints;
	}
    }

    // wypata gotwki
    private void debitLoyalty(APDU apdu) throws ISOException
    {
	// sprawdzenie czy wczeniej podano prawidowy kod PIN i czy terminal by uwierzytelniony 
        if (!ownerPIN.isValidated() || !externalAuth)
            ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);

        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
	
	// sprawdzenie poprawnoci numeru partnera
	if (apduBuffer[ISO7816.OFFSET_P1] != 1 &&
	    apduBuffer[ISO7816.OFFSET_P1] != 2)
	    ISOException.throwIt(ISO7816.SW_INCORRECT_P1P2);

        // sprawdzenie czy dostarczono 2 bajty danych (ilo punktw) i czy udao si je odebra
        if(apduBuffer[4] != 2 || apdu.setIncomingAndReceive() != 2)
	    // operacja zakoczona z bdem
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH) ;

	// odczytanie wartoci transakcji (bajty 5 i 6 w przesaniej komendzie APDU)
        short newPoints = (short)(((apduBuffer[5] << 8 ) & (short)0xFF00) +
	                  ((apduBuffer[6]) & (short)0x00FF));

        // sprawdzenie czy ilo punktw jest poprawna
        if ((points[apduBuffer[ISO7816.OFFSET_P1]-1] >= newPoints) 
		&& (newPoints > 0))
        {
	    // uaktualnienie stanu programu danego partnera
	    points[apduBuffer[ISO7816.OFFSET_P1]-1] -= newPoints ;

	    // nowy stan programu zostanie przesany w odpowiedzi
	    apduBuffer[9] = (byte)(points[apduBuffer[ISO7816.OFFSET_P1]-1] >> 8);
	    apduBuffer[10] = (byte)points[apduBuffer[ISO7816.OFFSET_P1]-1];

	    // przesanie 2 bajtw odpowiedzi
	    apdu.setOutgoing(); 
	    apdu.setOutgoingLength((short)2);
	    apdu.sendBytes((short)9, (short)2);
        }
        else
	    // ilo punktw po operacji nie byaby poprawna
	    throw new ISOException (ISO7816.SW_DATA_INVALID) ;
    }

    // weryfikacja kodu PIN
    private void verifyPIN(APDU apdu)
    {
        if (!externalAuth)
            ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);
	// pobranie bufora z APDU
        byte[] buffer = apdu.getBuffer();
	
	// dugo kodu PIN
        byte pinLength = buffer[ISO7816.OFFSET_LC];

	// pobranie przekazanych danych
	if(apdu.setIncomingAndReceive()<pinLength) 
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        
	if(!ownerPIN.check(buffer, ISO7816.OFFSET_CDATA, pinLength))
        {
	    // sprawdzenie kodu PIN nie powiodo si
            byte triesRemaining = ownerPIN.getTriesRemaining();
	    // ilo pozostaych prb wynosi x (63 Cx)
            ISOException.throwIt((short)(SW_WRONG_PIN + triesRemaining));
        }
    }

    // zwraca liczb losow (8 bajtw)
    private void askRandom(APDU apdu)
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
            
        // wygenerowanie liczby losowej (za komend APDU)
	random.generateData(apduBuffer, (short)5, (short)8);

	// skopiowanie liczby losowej do bufora
	Util.arrayCopy(apduBuffer, (short)5, lastRandom, (short)0, (short)8);

	// ustawienie flagi oznaczajcej wygenerowanie liczby
	randomGen=true;

	// przejcie w tryb odpowiedzi
        apdu.setOutgoing(); 
	// dugo odpowiedzi
        apdu.setOutgoingLength((short)8); 

        // przesanie odpowiedzi APDU (8 bajtw od bajtu 5 z bufora)
        apdu.sendBytes((short)5, (short)8);
    }

    // uwierzytelnienie karty
    private void internalAuth(APDU apdu)
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();

	// oczekujemy 8 bajtw danych
	if(apdu.setIncomingAndReceive() != 8) 
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
	
        // ustawienie w tryb szyfrowania
        cipherDES.init(desKey, Cipher.MODE_ENCRYPT);
	
	// szyfrowanie 
	cipherDES.doFinal(apduBuffer, (short)5, (short)8, apduBuffer, (short)13);
	
	// przejcie w tryb odpowiedzi
        apdu.setOutgoing(); 
	// dugo odpowiedzi
        apdu.setOutgoingLength((short)8); 

        // przesanie odpowiedzi APDU (8 bajty od bajtu 13 z bufora)
        apdu.sendBytes((short)13, (short)8);
    }

    // uwierzytelnienie terminala
    private void externalAuth(APDU apdu)
    {
	// nie byo zapytania o liczb losow
	if (!randomGen) 
	    ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);

        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
	
	// oczekujemy 8 bajtw szyfrogramu
	if(apdu.setIncomingAndReceive() != 8) 
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
	
	// tymczasowy bufor 
	byte[] buffer = new byte[8];
        
	// liczba losowa ju nie jest wana (ochrona przed atakiem)
	randomGen=false;
	
	// deszyfrowanie 
        cipherDES.init(desKey, Cipher.MODE_DECRYPT);
	cipherDES.doFinal(apduBuffer, (short)5, (short)8, buffer, (short)0);

	// jeli po zdeszyfrowaniu szyfrogramu nie otrzymano ostatniej
	// liczby losowej - operacja nie powioda si
	if (Util.arrayCompare(buffer, (short)0, lastRandom, 
			     (short)0, (short)8) != (short)0)
	    ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);
	else
	    // w przeciwnym przypadku terminal jest uwierzytelniony
	    externalAuth=true;
    }
}
