// import niezbdnych klas
import opencard.core.service.SmartCard;
import opencard.core.service.CardRequest;
import opencard.core.service.CardServiceException;

import opencard.core.terminal.CardID;
import opencard.core.terminal.CommandAPDU;
import opencard.core.terminal.ResponseAPDU;
import opencard.core.terminal.CardTerminalException;

import opencard.opt.util.PassThruCardService;

import opencard.core.util.HexString;
import opencard.core.util.OpenCardPropertyLoadingException;

public class OCFExample2
{
    public static void main (String[] args)
    {
	// komenda APDU
	CommandAPDU  apdu;

	// odpowied APDU
	ResponseAPDU rpdu;
	
	// komenda "get challenge" (CLA INS P1 P2 LE)
	final byte[] GET_CHALLENGE={(byte) 0x00, (byte) 0x84, 
				    (byte) 0x00, (byte) 0x00, 
				    (byte) 0x08};

	try
	{
	    // uruchomienie usug zwizanych z kartami inteligentnymi
	    SmartCard.start();

	    // klasa reprezentuje kart na jak czeka aplikacja
	    CardRequest cr = new CardRequest(CardRequest.ANYCARD, 
		    null, PassThruCardService.class);
	    
	    // ustawienie czasu oczekiwania
	    cr.setTimeout(0);

	    // oczekiwanie na woenie karty (w sekundach)
	    SmartCard sc = SmartCard.waitForCard(cr);

	    if (sc != null) 
	    {
		// klasa obsugujca sesj z kart
		PassThruCardService ptcs = (PassThruCardService) 
			sc.getCardService(PassThruCardService.class, true);
		
		// pobranie informacji o karcie
		CardID cid = sc.getCardID ();
		
		// wydruk ATR karty
		System.out.println ("ATR : " + 
			HexString.hexify(cid.getATR()));

		// przesanie komendy do karty
		apdu = new CommandAPDU(GET_CHALLENGE);
		rpdu = ptcs.sendCommandAPDU(apdu);
		
		// wydruk otrzymanych wynikw
		System.out.println("Response data : " + 
			HexString.hexify(rpdu.data()));

		System.out.println("Response SW : " + 
			HexString.hexifyShort(rpdu.sw()));
	    }
	    else
		System.out.println("Card not inserted!");
	    // zamknicie usug zwizanych z kartami inteligentnymi
	    SmartCard.shutdown();
	}
	// wyjtek zwizany z usugami systemowymi
	catch (CardServiceException ex) 
	{
            System.out.println("CardServiceException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany przy bdach terminala
	catch (CardTerminalException ex) 
	{
            System.out.println("CardTerminalException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany w sytuacji nieprawidowego 
	// pliku konfiguracyjnego lub jego braku
	catch (OpenCardPropertyLoadingException ex) 
	{
            System.out.println("OpenCardPropertyLoadingException : ");
            System.out.println(ex.getMessage());
        }
	// w przypadku braku dostpu do odpowiednich klas
	catch (ClassNotFoundException ex) 
	{
            System.out.println("ClassNotFoundException : ");
            System.out.println(ex.getMessage());
        } 
    } 
}

