// import niezbdnych klas
import opencard.core.service.SmartCard;
import opencard.core.service.CardServiceException;

import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.SlotChannel;
import opencard.core.terminal.CardID;
import opencard.core.terminal.CardTerminalException;
import opencard.core.terminal.InvalidSlotChannelException;

import opencard.core.event.CTListener;
import opencard.core.event.CardTerminalEvent;
import opencard.core.event.EventGenerator;

import opencard.core.util.HexString;
import opencard.core.util.OpenCardPropertyLoadingException;

public class OCFExample3
{
    // obiekt synchronizujcy zdarzenia
    private static Object SCMonitor = "Smart Card Event";

    // implementacja interfejsu CTListener
    static class CardListener implements CTListener
    {
	public void cardInserted (CardTerminalEvent ctEvent) 
	{
	    System.out.println ("Card inserted.");

	    try
	    {
		// pobranie instancji terminala
		CardTerminal terminal = ctEvent.getCardTerminal();

		// pobranie numeru slotu
		int slotID = ctEvent.getSlotID();

		// otwarcie slotu
		SlotChannel sch=terminal.openSlotChannel(slotID);

		// informacje o karcie 
		CardID cid = sch.getCardID();

		// wydruk ATR karty
		System.out.println ("ATR : " + 
			HexString.hexify(cid.getATR()));
	    }
	    // wyjtek zgaszany przy bdach slotu
	    catch (InvalidSlotChannelException ex) 
	    {
		System.out.println("InvalidSlotChannelException : ");
		System.out.println(ex.getMessage());
	    }
	    // wyjtek zgaszany przy bdach terminala
	    catch (CardTerminalException ex) 
	    {
		System.out.println("CardTerminalException : ");
		System.out.println(ex.getMessage());
	    }

	    // zgoszenie zdarzenia
	    synchronized (SCMonitor) 
	    { 
		SCMonitor.notifyAll(); 
	    }
	}

	public void cardRemoved (CardTerminalEvent ctEvent) 
	{
	    System.out.println("Card removed.");

	    // zgoszenie zdarzenia
	    synchronized (SCMonitor) 
	    {
		SCMonitor.notifyAll();
	    }
	}
    }
    
    public static void main (String[] args)
    {
	try
	{
	    // uruchomienie usug zwizanych z kartami inteligentnymi
	    SmartCard.start();

	    // rejestracja zdarzenia
	    EventGenerator.getGenerator().addCTListener(new CardListener());

	    synchronized (SCMonitor) 
	    {
		System.out.println("Waiting for event...");
		try
		{
		    // oczekiwanie na zdarzenie woenia lub wyjcia karty
		    SCMonitor.wait();
		}
		catch (InterruptedException ex)
		{
		    System.out.println("InterruptedException : ");
		    System.out.println(ex.getMessage());
		}
	    }

	    SmartCard.shutdown();
	}
	// wyjtek zwizany z usugami systemowymi
	catch (CardServiceException ex) 
	{
            System.out.println("CardServiceException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany przy bdach terminala
	catch (CardTerminalException ex) 
	{
            System.out.println("CardTerminalException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany w sytuacji nieprawidowego 
	// pliku konfiguracyjnego lub jego braku
	catch (OpenCardPropertyLoadingException ex) 
	{
            System.out.println("OpenCardPropertyLoadingException : ");
            System.out.println(ex.getMessage());
        }
	// w przypadku braku dostpu do odpowiednich klas
	catch (ClassNotFoundException ex) 
	{
            System.out.println("ClassNotFoundException : ");
            System.out.println(ex.getMessage());
        } 
    } 
}

