// import niezbdnych bibliotek
import com.linuxnet.jpcsc.*;
import java.io.*;

public class PCSCExample
{
    public static void main(String[] args)
    {
	// podczenie do zarzdcy zasobw PC/SC
	System.out.println("Connecting PC/SC Manager...");
	Context ctx = new Context();
	ctx.EstablishContext(PCSC.SCOPE_SYSTEM, null, null);

	// pobranie listy czytnikw dostpnych w systemie
	System.out.println("Readers list :");
	String[] readers = ctx.ListReaders();

	// wydruk listy czytnikw
	if (readers.length == 0)
	{
	    System.out.println("not found");
	    System.exit(0);
	}
	for (int i = 0; i < readers.length; i++)
	{
	    System.out.println("["+Integer.toString(i+1)+"] " + readers[i]);
	}

	// wybr czynnika
	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	int reader = 0;
	do
	{
	    try
	    {
		System.out.print("Select reader : ");
		String readerStr = in.readLine();
		reader = Integer.parseInt(readerStr);	
	    }
	    catch (Exception e) 
	    {
	    }
	}
	while (reader<1 || reader>readers.length);
	reader--;
	
	// oczekiwanie na woenie karty do czytnika
	System.out.println("Waiting for card in " + readers[reader] + "...");
	State[] states = new State[1];
	states[0] = new State(readers[reader]);
	// oczekiwanie na zmian statusu
	do
	{
	    ctx.GetStatusChange(1000, states);
	}
	while ((states[0].dwEventState & PCSC.STATE_PRESENT) != PCSC.STATE_PRESENT);
	
	System.out.println("Reader state : ");
	System.out.println(states[0]);
	    
	// poczenie do karty
	System.out.println("Connecting to the card...");
	Card card = ctx.Connect(readers[reader], PCSC.SHARE_EXCLUSIVE, 
				PCSC.PROTOCOL_T1 | PCSC.PROTOCOL_T0);
	    
	// status karty
	System.out.print("Card status : ");
	State cardSatus = card.Status();
	System.out.println(cardSatus.toString());
	
	// przesanie komendy APDU
	Apdu apdu = new Apdu(256);
	try
	{
	    // komenda "get challenge" (CLA INS P1 P2 LE)
	    final byte[] GET_CHALLENGE={(byte) 0x00, (byte) 0x84,
	                                (byte) 0x00, (byte) 0x00,
	                                (byte) 0x08};                                                 
	    byte[] resp;

	    apdu.set(GET_CHALLENGE);
	    System.out.println("APDU : " + apdu);
	    resp = card.Transmit(apdu);
	    // wydruk odpowiedzi
	    System.out.println("RAPDU : " + Apdu.ba2s(resp, 0, resp.length));
	}
	catch(PCSCException e)
	{
	    // wydruk informacji o zaistniaym bdzie
	    System.out.println("Exception! ("+e.getReason()+", "+ e.toString());
	}
	    
	// zakoczenie poczenia z kart
	System.out.println("Disconnecting the card...");
	card.Disconnect(PCSC.LEAVE_CARD);
	
	// zakoczenie poczenia z zarzdc zasobw PC/SC
	System.out.println("Disconnecting PC/SC manager...");
	ctx.ReleaseContext();
    }
}
